declare namespace MextPlugins {
  /**
   * 使用 Consumer 获得全局 store
  ```
  <Consumer>
    {(store) => {
      console.log('the global store: ', store);
      return <YourComponent />;
    }}
  </Consumer>
    ```
   */
  export const Consumer: any;

  /**
   * 设置meta信息
    ```
    <Head>
      <title>这是标题</title>
    </Head>
    ```
   */
  export const Head: any;

  /**
   *    启动react渲染,只能执行一次，就算后续执行了也不会启任何作用，并且会在控制台进行错误提示
   *    @Component       项目启动入口组件
   *    @ErrorComponent  展示错误的组件
   */
  export const start: (Component: any, ErrorComponent?: any) => void;

  /**
   * setMext是mext内置的一个迷你数据管理, 方便解决数据复杂度不大, 业务逻辑相对简单的场景的数据管理需求
   *
   * ```
   * setMext({ name: Math.random() });
   * ```
   */
  export const setMext: (data: { [key: string]: any }) => void;

  /**
   * 删除 name 字段
   *
   * `removeMext('name');`
   */
  export const removeMext: (data: string) => void;

  /**
   * 获取basename
   */
  export const basename: () => string;
}

/**
 *
 * `start` 函数，启动mext应用
 *
 * `Head` 组件，设置header部分内容
 *
 * `Consumer` 组件，使用setMext设置的数据
 *
 * `setMext` 函数，设置store
 *
 * `removeMext` 函数，移除指定的store
 *
 * `basename` 函数，获取项目的basename
 *
 * `ErrorProps` TS类型，指定错误组件的props的ts描述
 *
 *
 */
declare module 'mext' {
  import Consumer from 'mext/lib/consumer';
  import { Helmet as Head } from 'react-helmet';
  import start, { ErrorProps } from 'mext/lib/start';
  import setMext from 'mext/lib/setMext';
  import removeMext from 'mext/lib/removeMext';
  export default MextPlugins;
  const basename: () => string;
  export { Head, Consumer, start, setMext, removeMext, basename, ErrorProps };
}
