'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

class EventEmitter {
  constructor() {
    this.listeners = {};
  }

  on(event, cb) {
    if (!this.listeners[event]) {
      this.listeners[event] = new Set();
    }

    if (this.listeners[event].has(cb)) {
      throw new Error(`The listener already exising in event: ${event}`);
    }

    this.listeners[event].add(cb);
  }

  emit(event, ...data) {
    if (!this.listeners[event]) {
      return false;
    }

    return this.listeners[event].forEach(cb => cb(...data));
  }

  exist(event) {
    if (!this.listeners[event]) {
      return false;
    }

    return true;
  }

  off(event) {
    delete this.listeners[event];
  }
}

exports.default = EventEmitter;
