'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

const React = require('react');

const Exception_1 = require('mext-utils/Exception');

const client_1 = require('mext-plugin/client');
/**
 *    启动react渲染,只能执行一次，就算后续执行了也不会启任何作用，并且会在控制台进行错误提示
 *    @Component       项目启动入口组件
 *    @ErrorComponent  展示错误的组件
 */

function start(
  Component,
  ErrorComponent, // options主要通过babel注入的一些配置信息，目前有plugins
  options = {}
) {
  let RootComponent = Component;

  if (React.isValidElement(Component)) {
    RootComponent = props =>
      React.cloneElement(Component, typeof Component.type === 'function' ? { ...props } : {});
  }

  if (ErrorComponent) {
    Exception_1.capture(ErrorComponent);
  }

  if (process.env.RUN_ENV === 'node') {
    // node端，通过react-dom/server进行服务端渲染
    global.AppRegistry(RootComponent);
  } else {
    // 注册插件
    if (options && options.plugins && options.plugins.length) {
      client_1.default.register(options.plugins);
    } // 客户端渲染应用

    let _start = require('./web');

    if (process.env.NODE_ENV === 'development') {
      return async hot => {
        await _start(RootComponent, hot);
      };
    } else {
      return _start(RootComponent);
    }
  }
}

exports.default = start;
